import $ from 'jquery';
import * as d3 from 'd3'

window.jQuery = $;
window.$ = $;


export function DrawChart(Data,
    containerId,
    tootltipId,
    maxYDomain,
    Ylabel,
    YlabelSeparation,
    Yticks,
    customWidth,
    customHeight,
    colorRangeBars,
    colorRangeLines,
    colorRangeTooltip,
    highlightedBarName,
    highlightedBarColor,
    documentFontSizePx,
    threshold
) {
    $("#" + containerId).html("");

    var fontFactor = documentFontSizePx / 14.0;

    var length = Data.length;
    if(containerId=='estadisticas_carrera'){
        if(length<2){
            length = length * 3.5;
        }else if(length<4){
            length = length * 2.5;
        }else if(length<7){
            length = length * 1.5;
        }else if(length<10){
            length = length * 1.1;
        }
    }

    var margin = { top: 20 * fontFactor, right: 30 * fontFactor, bottom: 60 * fontFactor, left: 60 * fontFactor },
        width = customWidth * fontFactor * length - margin.left - margin.right,
        height = customHeight * fontFactor - margin.top - margin.bottom;

	if(containerId=='estadisticas_carrera'){
		if (width<=330){
			width = 330;
		}
	}
    // Define the div for the tooltip
    var tooltipDiv = d3.select("#" + tootltipId).append("div")
        .attr("class", "tooltip")
        .style("opacity", 0);

    function SetTooltipData(element, dataItem, bgColor, isStyle, scalableStyle, scaleStart, scaleEnd) {
        element.on("mouseover", function (dt) {

            var offsetX = d3.event.offsetX ;
            for (var i = 0; i < scalableStyle.length; i++) {
                if (isStyle)
                    element.style(scalableStyle[i], scaleEnd[i]);
                else
                    element.attr(scalableStyle[i], scaleEnd[i]);
            }
            tooltipDiv.style("background", bgColor)
            tooltipDiv.transition()
                .duration(200)
                .style("opacity", .9);
            if(dt.Name=="Rango de Promedios"){
                offsetX = offsetX + (window.innerWidth/3) - 28;
            }else if(dt.Name=="Carga Horaria Semanal" || dt.Name=="Total"){
                offsetX = offsetX + (window.innerWidth/2) - 28;
            }
            tooltipDiv.html(dataItem.Tooltip_str)
                .style("left", ( offsetX + "px"))
                .style("top", (d3.event.pageY - document.body.scrollTop - 28) + "px");
        })
            .on("mouseout", function (d) {
                for (var i = 0; i < scalableStyle.length; i++) {
                    if (isStyle)
                        element.style(scalableStyle[i], scaleStart[i]);
                    else
                        element.attr(scalableStyle[i], scaleStart[i]);
                }
                tooltipDiv.transition()
                    .duration(500)
                    .style("opacity", 0);
            });
    }

    function getTextWidth(text, fontSize, fontName) {
        let c = document.createElement("canvas");
        let ctx = c.getContext("2d");
        ctx.font = fontSize + ' ' + fontName;
        return ctx.measureText(text).width;
    }

    function DataSegregator(array, on) {
        var SegData;
        let OrdinalPositionHolder = {
            valueOf: function () {
                let thisObject = this;
                let keys = Object.keys(thisObject);
                keys.splice(keys.indexOf("valueOf"), 1);
                keys.splice(keys.indexOf("keys"), 1);
                return keys.length == 0 ? -1 : d3.max(keys, function (d) { return thisObject[d] })
            }
            , keys: function () {
                let thisObject = this
                let keys = Object.keys(thisObject);
                keys.splice(keys.indexOf("valueOf"), 1);
                keys.splice(keys.indexOf("keys"), 1);
                return keys;
            }
        }
        array[0].map(function (d) { return d[on] }).forEach(function (b) {
            let value = OrdinalPositionHolder.valueOf();
            OrdinalPositionHolder[b] = OrdinalPositionHolder > -1 ? ++value : 0;
        })

        SegData = OrdinalPositionHolder.keys().map(function () {
            return [];
        });

        array.forEach(function (d) {
            d.forEach(function (b) {
                SegData[OrdinalPositionHolder[b[on]]].push(b);
            })
        });

        return SegData;
    }

    var textWidthHolder = 0;
    /// Adding Date in LineCategory
    Data.forEach(function (d) {
        d.LineCategory.forEach(function (b) {
            b.Date = d.Date;
        })
    });

    var Categories = new Array();
    // Extension method declaration

    var Data;
    var ageNames;
    var x0 = d3.scaleBand()
        .rangeRound([0, width], .1);
    var XLine = d3.scaleBand()
        .rangeRound([0, width], .1);
    var x1 = d3.scaleBand();

    var y = d3.scaleLinear()
        .range([height, 0]);

    var YLine = d3.scaleLinear().range([height, 0])
        //.domain([0, d3.max(Data, function (d) { return d3.max(d.LineCategory, function (b) { return b.Value }) })]);
        .domain([0, 10]);

    var color = d3.scaleOrdinal()
        .range(colorRangeBars);
    var colorToolTip = d3.scaleOrdinal()
        .range(colorRangeTooltip);

    var line = d3.line().x(function (d) {
        return x0(d.Date) + x0.bandwidth() / 2;
    }).y(function (d) { return YLine(d.Value) });


    var xAxis = d3.axisBottom()
        .scale(x0)

    var yAxis = d3.axisLeft()
        .scale(y)
        .ticks(Yticks)
        .tickSize(-width, 0, 0)
        .tickFormat(d3.format(".2s"));

    // var YLeftAxis = d3.svg.axis().scale(YLine).orient("right").tickFormat(d3.format(".2s"));

    var svg = d3.select("#" + containerId).append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", "translate(" + margin.left + "," + margin.top + ")");



    // Bar Data categories
    Data.forEach(function (d) {
        d.Categories.forEach(function (b) {
            if (Categories.findIndex(function (c) { return c.Name === b.Name }) == -1) {
                b.Type = "bar";
                //console.log(JSON.stringify(b))
                Categories.push(b)
            }
        })
    });


    // Line Data categories
    Data.forEach(function (d) {
        d.LineCategory.forEach(function (b) {
            if (Categories.findIndex(function (c) { return c.Name === b.Name }) == -1) {
                b.Type = "line";
                Categories.push(b)
            }
        })
    });

    // Processing Line data
    let lineData = Data[0].LineCategory.length == 0 ? [] : DataSegregator(Data.map(function (d) { return d.LineCategory }), "Name");

    // Line Coloring
    let LineColor = d3.scaleOrdinal();
    LineColor.domain(Categories.filter(function (d) { return d.Type == "line" }).map(function (d) { return d.Name }));
    LineColor.range(colorRangeLines)
    x0.domain(Data.map(function (d) { return d.Date; }));
    XLine.domain(Data.map(function (d) { return d.Date; }));
    x1.domain(Categories.filter(function (d) { return d.Type == "bar" }).map(function (d) { return d.Name })).range([x0.bandwidth() * 0.2, x0.bandwidth() * 0.8]);
    y.domain([0, maxYDomain]);

    svg.append("g")
        .attr("class", "x axis")
        .attr("transform", "translate(0," + height + ")")
        .call(xAxis);


    svg.append("g")
        .attr("class", "y axis")
        .call(yAxis)
        .append("text")
        .attr("transform", "rotate(-90)")
        .attr("y", YlabelSeparation) //-50
        .attr("x", - height / 2 + 50)
        .attr("dy", ".71em")
        .style("text-anchor", "end")
        .text(Ylabel);

    var state = svg.selectAll(".state")
        .data(Data)
        .enter().append("g")
        .attr("class", "state")
        .attr("transform", function (d) { return "translate(" + x0(d.Date) + ",0)"; });

    state.selectAll("rect")
        .data(function (d) { return d.Categories; })
        .enter().append("rect")
        .attr("width", x1.bandwidth())
        .attr("x", function (d) {
            return x1(d.Name);
        })
        .attr("y", function (d) {
            var element = d3.select(this);
            SetTooltipData(element, d, colorToolTip(d.Name), false,
                ["width", "x"],
                [element.attr("width"), element.attr("x")],
                [element.attr("width") * 1.25, element.attr("x") - element.attr("width") * 0.25 / 2.0]);
            return y(d.Value);
        })
        //.attr("height", function (d) { return height - y(d.Value); })
        .style("fill", function (d) {
            if (d.Bar == highlightedBarName)
                return highlightedBarColor;
            else
                return color(d.Name);
        })
        .transition().delay(200).attrTween("height", function (d) {
            var i = d3.interpolate(0, height - y(d.Value));
            return function (t) {
                return i(t);
            }
        });


    // drawaing lines
    var index = 0;

    var line2 = d3.line().x(function (d) {
        return x0(d.Date) + x0.bandwidth() / 2;
    }).y(function (d) { return YLine(d.Value) });
    svg.selectAll(".lines").data(lineData).enter().append("g").attr("class", "line")
        .each(function (d) {
            let Name = d[0].Name
            var obj = d3.select(this);
            var path = obj.append("path");
            path.attr("stroke", LineColor(Name))
               .attr("stroke-linejoin", "round")
               .attr("stroke-linecap", "round")
               .attr("stroke-width", '0.3em')
               .attr("fill", 'none')
               .attr("id", Name)
               .attr("d", function (b) { return line(b) }).transition().duration(1500);

            for (var i = 0; i < d.length; i++) {
                var px = x0(d[i].Date) + x0.bandwidth() / 2;
                var py = YLine(d[i].Value);
                var item = d[i];
                var circle = obj.append("circle");
                circle.attr("cx", px)
                    .attr("cy", py)
                    .attr("r", height * 0.04)
                    .attr("fill", LineColor(Name));
                SetTooltipData(circle, item, "#fffb99", false, ["r"], [height * 0.04], [height * 0.06]);
            }
        });

    //Threshold line
    if(threshold !== null){
        svg.append("g")
            .attr("transform", "translate(0, " + y(threshold.value) + ")")
            .append("line")
            .attr("x2", width)
            .style("stroke", threshold.color)
            .style("stroke-width", threshold.width);
    }


    // Legends


    var LegendHolder = svg.append("g").attr("class", "legendHolder");
    var legend = LegendHolder.selectAll(".legend")
        .data(Categories.map(function (d) { return { "Name": d.Name, "Type": d.Type } }))
        .enter().append("g")
        .attr("class", "legend")
        .attr("transform", function (d, i) { return "translate(0," + (height + margin.bottom / 2) + ")"; })
        .each(function (d, i) {
            //  Legend Symbols


            d3.select(this).append("rect")
                .attr("width", function () { return 18 })
                .attr("x", function (b) {

                    let left = (i + 1) * 15 + i * 18 + i * 5 + textWidthHolder;
                    return left;
                })
                .attr("y", function (b) { return b.Type == 'bar' ? 0 : 7 })
                .attr("height", function (b) { return b.Type == 'bar' ? 18 : 5 })
                .style("fill", function (b) { return b.Type == 'bar' ? color(d.Name) : LineColor(d.Name) });

            //  Legend Text

            d3.select(this).append("text")
                .attr("x", function (b) {

                    let left = (i + 1) * 15 + (i + 1) * 18 + (i + 1) * 5 + textWidthHolder;

                    return left;
                })
                .attr("y", 9)
                .attr("dy", ".35em")
                .style("text-anchor", "start")
                .text(d.Name);
            textWidthHolder += getTextWidth(d.Name, d3.select(this).style("font-size"), "calibri");
        });



    // Legend Placing

    d3.select("#" + containerId).select(".legendHolder").attr("transform", function (d) {
        let thisWidth = d3.select(this).node().getBBox().width;
        return "translate(" + ((width) / 2 - thisWidth / 2) + ",0)";
    })
}
