import $ from 'jquery';
import * as d3 from 'd3'

window.jQuery = $;
window.$ = $;

export function DrawHorizontalBarChart(data,
    containerId,
    tootltipId,
    customWidth,
    customHeight,
    documentFontSizePx,
    xLabel) {
    $("#" + containerId).html("");

    // //sort bars based on value
    // data = data.sort(function (a, b) {
    //     return d3.ascending(a.value, b.value);
    // })

    data = data.reverse();

    //set up svg using margin conventions - we'll need plenty of room on the left for labels

    var fontFactor = documentFontSizePx / 14.0;

    var margin = {
        top: 10 * fontFactor,
        right: 25 * fontFactor,
        bottom: 40 * fontFactor,
        left: 160 * fontFactor
    };

    var width = customWidth * fontFactor - margin.left - margin.right,
        height = customHeight * fontFactor - margin.top - margin.bottom;

    var svg = d3.select("#" + containerId).append("svg")
        .attr("width", width + margin.left + margin.right)
        .attr("height", height + margin.top + margin.bottom)
        .append("g")
        .attr("transform", "translate(" + margin.left + "," + margin.top + ")");


    // Define the div for the tooltip
    var tooltipDiv = d3.select("#" + tootltipId).append("div")
        .attr("class", "tooltipH")
        .style("opacity", 0);

    function SetTooltipData(element, tootltipContent) {
        element.on("mouseover", function (dt) {
            tooltipDiv.transition()
                .duration(200)
                .style("opacity", .98);
            tooltipDiv.html(tootltipContent)
                .style("left", (d3.event.pageX) + "px")
                .style("top", (d3.event.pageY - document.body.scrollTop - 28) + "px");
        })
            .on("mouseout", function (d) {
                tooltipDiv.transition()
                    .duration(500)
                    .style("opacity", 0);
            });
    }


    var x = d3.scaleLinear()
        .range([0, width])
        .domain([0, 1]);
    // .domain([0, d3.max(data, function (d) {
    //     return d.value;
    // })]);

    var y = d3.scaleBand()
        .rangeRound([height, 0], .4)
        .domain(data.map(function (d) {
            return d.name;
        }));

    //make y axis to show bar names
    var yAxis = d3.axisLeft(y).tickSize(0);

    var xAxis = d3.axisBottom(x)
        .scale(x)
        .ticks(5)
        .tickSize(-height, 0, 0)
        .tickFormat(d3.format(",.2f"))

    svg.append("g")
        .attr("class", "x haxis")
        .attr("transform", "translate(0," + height + ")")
        .call(xAxis);

    var gy = svg.append("g")
        .attr("class", "y haxis")
        .call(yAxis)
        .selectAll(".tick text")
        .call(function (text) {
            text.each(function () {
                var text = d3.select(this);
                var content = text.text();
                text.text(content.substring(0, 21) + (content.length > 21 ? ".. " : ""));
            });
        });

    var bars = svg.selectAll(".barH")
        .data(data)
        .enter()
        .append("g")

    //append rects
    bars.append("rect")
        .attr("class", "barH")
        .attr("y", function (d) {
            SetTooltipData(d3.select(this),d.tooltip);
            return y(d.name);
        })
        .attr("height", y.bandwidth()-7)
        .attr("x", 0)
        // .attr("width", function (d) {
        //     return x(d.value);
        // })
        .transition().delay(200).attrTween("width", function (d) {
            var i = d3.interpolate(0, x(d.value));
            return function (t) {
                return i(t);
            }
        });

    // //add a value label to the right of each bar
    // bars.append("text")
    //     .attr("class", "label")
    //     //y position of the label is halfway down the bar
    //     .attr("y", function (d) {
    //         return y(d.name) + y.rangeBand() / 2 + 4;
    //     })
    //     //x position is 3 pixels to the right of the bar
    //     .attr("x", function (d) {
    //         return x(d.value) + 3;
    //     })
    //     .text(function (d) {
    //         return d.value;
    //     });



    // Legends

    function getTextWidth(text, fontSize, fontName) {
        let c = document.createElement("canvas");
        let ctx = c.getContext("2d");
        ctx.font = fontSize + ' ' + fontName;
        return ctx.measureText(text).width;
    }

    var textWidthHolder = 0;
    var LegendHolder = svg.append("g").attr("class", "legendHolder");
    var legend = LegendHolder.selectAll(".legend")
        .data([{ "Name": xLabel }])
        .enter().append("g")
        .attr("class", "legend")
        .attr("transform", function (d, i) { return "translate(0," + (height + margin.bottom * 0.5) + ")"; })
        .each(function (d, i) {
            //  Legend Symbols


            d3.select(this).append("rect")
                .attr("width", function () { return 18 })
                .attr("x", function (b) {

                    let left = (i + 1) * 15 + i * 18 + i * 5 + textWidthHolder;
                    return left;
                })
                .attr("y", function (b) { return 0; })
                .attr("height", function (b) { return 18; })
                .style("fill", function (b) { return d3.select(".barH").style("fill"); });

            //  Legend Text

            d3.select(this).append("text")
                .attr("x", function (b) {

                    let left = (i + 1) * 15 + (i + 1) * 18 + (i + 1) * 5 + textWidthHolder;

                    return left;
                })
                .attr("y", 9)
                .attr("dy", ".35em")
                .style("text-anchor", "start")
                .text(d.Name);
            textWidthHolder += getTextWidth(d.Name, d3.select(this).style("font-size"), "calibri");
        });



    // Legend Placing

    d3.select("#" + containerId).select(".legendHolder").attr("transform", function (d) {
        let thisWidth = d3.select(this).node().getBBox().width;
        return "translate(" + ((width) / 2 - thisWidth / 2) + ",0)";
    })
}


export function wrap(text, width) {
    text.each(function () {
        var text = d3.select(this),
            words = text.text().split(/\s+/).reverse(),
            word,
            line = [],
            lineNumber = 0,
            lineHeight = 1.1, // ems
            y = text.attr("y"),
            dy = parseFloat(text.attr("dy")),
            tspan = text.text(null).append("tspan").attr("x", 0).attr("y", y).attr("dy", dy + "em");
        while (word = words.pop()) {
            line.push(word);
            tspan.text(line.join(" "));
            if (tspan.node().getComputedTextLength() > width) {
                line.pop();
                tspan.text(line.join(" "));
                line = [word];
                tspan = text.append("tspan").attr("x", 0).attr("y", y).attr("dy", ++lineNumber * lineHeight + dy + "em").text(word);
            }
        }
    });
}
